@@ -18,19 +18,21 @@ package network
 
 import (
 	"bytes"
+	"errors"
 	"net/http"
 	"net/http/httptest"
+	"reflect"
 	"testing"
 	"text/template"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
-	"github.com/knative/pkg/system"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"knative.dev/pkg/system"
 
-	. "github.com/knative/pkg/configmap/testing"
-	_ "github.com/knative/pkg/system/testing"
+	. "knative.dev/pkg/configmap/testing"
+	_ "knative.dev/pkg/system/testing"
 )
 
 func TestOurConfig(t *testing.T) {
@@ -45,7 +47,8 @@ func TestOurConfig(t *testing.T) {
 }
 
 func TestConfiguration(t *testing.T) {
-	nonDefaultDomainTemplate := "{{.Namespace}}.{{.Name}}.{{.Domain}}"
+	const nonDefaultDomainTemplate = "{{.Namespace}}.{{.Name}}.{{.Domain}}"
+	ignoreDT := cmpopts.IgnoreFields(Config{}, "DomainTemplate")
 
 	networkConfigTests := []struct {
 		name       string
@@ -56,11 +59,12 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with no network input",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -84,10 +88,11 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with empty network",
 		wantErr: false,
 		wantConfig: &Config{
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -162,10 +167,11 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with invalid network string",
 		wantErr: false,
 		wantConfig: &Config{
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -180,10 +186,11 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with invalid network string",
 		wantErr: false,
 		wantConfig: &Config{
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -198,10 +205,11 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with invalid network range",
 		wantErr: false,
 		wantConfig: &Config{
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -216,11 +224,12 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with valid CIDR network range",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "10.10.10.0/24",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "10.10.10.0/24",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -235,11 +244,12 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with multiple valid network ranges",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "10.10.10.0/24,10.240.10.0/14,192.192.10.0/16",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "10.10.10.0/24,10.240.10.0/14,192.192.10.0/16",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -254,11 +264,12 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with valid network",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -273,11 +284,12 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with non-Istio ingress type",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "foo-ingress",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "foo-ingress",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -285,19 +297,41 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
+			},
+		},
+	}, {
+		name:    "network configuration with non-Cert-Manager Certificate type",
+		wantErr: false,
+		wantConfig: &Config{
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: "foo-cert",
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
+		},
+		config: &corev1.ConfigMap{
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: system.Namespace(),
+				Name:      ConfigName,
+			},
+			Data: map[string]string{
+				IstioOutboundIPRangesKey:   "*",
+				DefaultCertificateClassKey: "foo-cert",
 			},
 		},
 	}, {
 		name:    "network configuration with diff domain template",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "foo-ingress",
-			DomainTemplate:             nonDefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "foo-ingress",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          nonDefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -305,9 +339,9 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
-				DomainTemplateKey:             nonDefaultDomainTemplate,
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
+				DomainTemplateKey:        nonDefaultDomainTemplate,
 			},
 		},
 	}, {
@@ -319,9 +353,9 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
-				DomainTemplateKey:             "",
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
+				DomainTemplateKey:        "",
 			},
 		},
 	}, {
@@ -333,8 +367,8 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
 				// This is missing a closing brace.
 				DomainTemplateKey: "{{.Namespace}.{{.Name}}.{{.Domain}}",
 			},
@@ -348,8 +382,8 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
 				// This is missing a closing brace.
 				DomainTemplateKey: "{{.Namespace}.{{.Name}}.{{.Domain}}",
 			},
@@ -363,8 +397,8 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
 				// Paths are disallowed
 				DomainTemplateKey: "{{.Domain}}/{{.Namespace}}/{{.Name}}.",
 			},
@@ -378,8 +412,8 @@ func TestConfiguration(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
 				// Bad variable
 				DomainTemplateKey: "{{.Name}}.{{.NAmespace}}.{{.Domain}}",
 			},
@@ -388,12 +422,13 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with Auto TLS enabled",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			AutoTLS:                    true,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			AutoTLS:                 true,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -409,12 +444,13 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with Auto TLS disabled",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			AutoTLS:                    false,
-			HTTPProtocol:               HTTPEnabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			AutoTLS:                 false,
+			HTTPProtocol:            HTTPEnabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -430,12 +466,13 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with HTTPProtocol disabled",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			AutoTLS:                    true,
-			HTTPProtocol:               HTTPDisabled,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			AutoTLS:                 true,
+			HTTPProtocol:            HTTPDisabled,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -452,12 +489,13 @@ func TestConfiguration(t *testing.T) {
 		name:    "network configuration with HTTPProtocol redirected",
 		wantErr: false,
 		wantConfig: &Config{
-			IstioOutboundIPRanges:      "*",
-			DefaultClusterIngressClass: "istio.ingress.networking.knative.dev",
-			DomainTemplate:             DefaultDomainTemplate,
-			TagTemplate:                DefaultTagTemplate,
-			AutoTLS:                    true,
-			HTTPProtocol:               HTTPRedirected,
+			IstioOutboundIPRanges:   "*",
+			DefaultIngressClass:     "istio.ingress.networking.knative.dev",
+			DefaultCertificateClass: CertManagerCertificateClassName,
+			DomainTemplate:          DefaultDomainTemplate,
+			TagTemplate:             DefaultTagTemplate,
+			AutoTLS:                 true,
+			HTTPProtocol:            HTTPRedirected,
 		},
 		config: &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{
@@ -494,8 +532,6 @@ func TestConfiguration(t *testing.T) {
 				t.Errorf("DomainTemplate(data) = %s, wanted %s", got, want)
 			}
 
-			ignoreDT := cmpopts.IgnoreFields(Config{}, "DomainTemplate")
-
 			if diff := cmp.Diff(actualConfig, tt.wantConfig, ignoreDT); diff != "" {
 				t.Fatalf("want %v, but got %v",
 					tt.wantConfig, actualConfig)
@@ -521,9 +557,9 @@ func TestAnnotationsInDomainTemplate(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
-				DomainTemplateKey:             `{{.Name}}.{{ index .Annotations "sub"}}.{{.Domain}}`,
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
+				DomainTemplateKey:        `{{.Name}}.{{ index .Annotations "sub"}}.{{.Domain}}`,
 			},
 		},
 		data: DomainTemplateValues{
@@ -542,9 +578,9 @@ func TestAnnotationsInDomainTemplate(t *testing.T) {
 				Name:      ConfigName,
 			},
 			Data: map[string]string{
-				IstioOutboundIPRangesKey:      "*",
-				DefaultClusterIngressClassKey: "foo-ingress",
-				DomainTemplateKey:             `{{.Name}}.{{.Namespace}}.{{.Domain}}`,
+				IstioOutboundIPRangesKey: "*",
+				DefaultIngressClassKey:   "foo-ingress",
+				DomainTemplateKey:        `{{.Name}}.{{.Namespace}}.{{.Domain}}`,
 			},
 		},
 		data: DomainTemplateValues{
@@ -589,7 +625,7 @@ func TestIsKubeletProbe(t *testing.T) {
 	if IsKubeletProbe(req) {
 		t.Error("Not a kubelet probe but counted as such")
 	}
-	req.Header.Set("User-Agent", kubeProbeUAPrefix+"1.14")
+	req.Header.Set("User-Agent", KubeProbeUAPrefix+"1.14")
 	if !IsKubeletProbe(req) {
 		t.Error("kubelet probe but not counted as such")
 	}
@@ -603,6 +639,66 @@ func TestIsKubeletProbe(t *testing.T) {
 	}
 }
 
+func TestKnativeProbeHeader(t *testing.T) {
+	req, err := http.NewRequest(http.MethodGet, "http://example.com/", nil)
+	if err != nil {
+		t.Fatalf("Error building request: %v", err)
+	}
+	if h := KnativeProbeHeader(req); h != "" {
+		t.Errorf("KnativeProbeHeader(req)=%v, want empty string", h)
+	}
+	want := "activator"
+	req.Header.Set(ProbeHeaderName, want)
+	if h := KnativeProbeHeader(req); h != want {
+		t.Errorf("KnativeProbeHeader(req)=%v, want %v", h, want)
+	}
+	req.Header.Set(ProbeHeaderName, "")
+	if h := KnativeProbeHeader(req); h != "" {
+		t.Errorf("KnativeProbeHeader(req)=%v, want empty string", h)
+	}
+}
+
+func TestKnativeProxyHeader(t *testing.T) {
+	req, err := http.NewRequest(http.MethodGet, "http://example.com/", nil)
+	if err != nil {
+		t.Fatalf("Error building request: %v", err)
+	}
+	if h := KnativeProxyHeader(req); h != "" {
+		t.Errorf("KnativeProxyHeader(req)=%v, want empty string", h)
+	}
+	want := "activator"
+	req.Header.Set(ProxyHeaderName, want)
+	if h := KnativeProxyHeader(req); h != want {
+		t.Errorf("KnativeProxyHeader(req)=%v, want %v", h, want)
+	}
+	req.Header.Set(ProxyHeaderName, "")
+	if h := KnativeProxyHeader(req); h != "" {
+		t.Errorf("KnativeProxyHeader(req)=%v, want empty string", h)
+	}
+}
+
+func TestIsProbe(t *testing.T) {
+	// Not a probe
+	req, err := http.NewRequest(http.MethodGet, "http://example.com/", nil)
+	if err != nil {
+		t.Fatalf("Error building request: %v", err)
+	}
+	if IsProbe(req) {
+		t.Error("Not a probe but counted as such")
+	}
+	// Kubelet probe
+	req.Header.Set("User-Agent", KubeProbeUAPrefix+"1.14")
+	if !IsProbe(req) {
+		t.Error("Kubelet probe but not counted as such")
+	}
+	// Knative probe
+	req.Header.Del("User-Agent")
+	req.Header.Set(ProbeHeaderName, "activator")
+	if !IsProbe(req) {
+		t.Error("Knative probe but not counted as such")
+	}
+}
+
 func TestRewriteHost(t *testing.T) {
 	r := httptest.NewRequest(http.MethodGet, "http://love.is/not-hate", nil)
 	r.Header.Set("Host", "love.is")
@@ -647,6 +743,96 @@ func TestRewriteHost(t *testing.T) {
 	}
 
 	if got, want := r.Header.Get(OriginalHostHeader), ""; got != want {
-		t.Errorf("r.Header[%s] = %q	, want: %q", OriginalHostHeader, got, want)
+		t.Errorf("r.Header[%s] = %q, want: %q", OriginalHostHeader, got, want)
+	}
+}
+
+func TestNameForPortNumber(t *testing.T) {
+	for _, tc := range []struct {
+		name       string
+		svc        *corev1.Service
+		portNumber int32
+		portName   string
+		err        error
+	}{{
+		name: "HTTP to 80",
+		svc: &corev1.Service{
+			Spec: corev1.ServiceSpec{
+				Ports: []corev1.ServicePort{{
+					Port: 80,
+					Name: "http",
+				}, {
+					Port: 443,
+					Name: "https",
+				}},
+			},
+		},
+		portName:   "http",
+		portNumber: 80,
+	}, {
+		name: "no port",
+		svc: &corev1.Service{
+			Spec: corev1.ServiceSpec{
+				Ports: []corev1.ServicePort{{
+					Port: 443,
+					Name: "https",
+				}},
+			},
+		},
+		portNumber: 80,
+		err:        errors.New("no port with number 80 found"),
+	}} {
+		t.Run(tc.name, func(t *testing.T) {
+			portName, err := NameForPortNumber(tc.svc, tc.portNumber)
+			if !reflect.DeepEqual(err, tc.err) { // cmp Doesn't work well here due to private fields.
+				t.Errorf("Err = %v, want: %v", err, tc.err)
+			}
+			if tc.err == nil && portName != tc.portName {
+				t.Errorf("PortName = %s, want: %s", portName, tc.portName)
+			}
+		})
+	}
+}
+
+func TestPortNumberForName(t *testing.T) {
+	for _, tc := range []struct {
+		name       string
+		subset     corev1.EndpointSubset
+		portNumber int32
+		portName   string
+		err        error
+	}{{
+		name: "HTTP to 80",
+		subset: corev1.EndpointSubset{
+			Ports: []corev1.EndpointPort{{
+				Port: 8080,
+				Name: "http",
+			}, {
+				Port: 8443,
+				Name: "https",
+			}},
+		},
+		portName:   "http",
+		portNumber: 8080,
+	}, {
+		name: "no port",
+		subset: corev1.EndpointSubset{
+			Ports: []corev1.EndpointPort{{
+				Port: 8443,
+				Name: "https",
+			}},
+		},
+		portName: "http",
+		err:      errors.New(`no port for name "http" found`),
+	}} {
+		t.Run(tc.name, func(t *testing.T) {
+			portNumber, err := PortNumberForName(tc.subset, tc.portName)
+			if !reflect.DeepEqual(err, tc.err) { // cmp Doesn't work well here due to private fields.
+				t.Errorf("Err = %v, want: %v", err, tc.err)
+			}
+			if tc.err == nil && portNumber != tc.portNumber {
+				t.Errorf("PortNumber = %d, want: %d", portNumber, tc.portNumber)
+			}
+		})
 	}
 }
